package com.jiuqi.util.inifile;

import java.io.OutputStream;
import java.io.InputStream;
import com.jiuqi.util.Stream;
import java.util.ArrayList;
import java.util.List;

public class BufferIni
{
    private List list;
    private IniBuffer buffer;
    private boolean founded;
    
    protected IniBuffer buffer() {
        return this.buffer;
    }
    
    public void readInfo() {
        this.list.clear();
        if (!this.buffer.reading() && !this.buffer.gotoEntry(true)) {
            return;
        }
        this.buffer.beginRead();
        try {
            for (int l = this.buffer.readInfoInteger(), i = 0; i < l; ++i) {
                final Section sec = new Section(this.buffer.readInfoString(), this.buffer);
                this.list.add(sec);
                sec.readInfo();
            }
        }
        finally {
            this.buffer.endRead();
        }
        this.buffer.endRead();
    }
    
    public void writeInfo() {
        this.buffer.beginWrite();
        try {
            final int l = this.count();
            this.buffer.writeInfoInteger(l);
            for (int i = 0; i < l; ++i) {
                final Section sec = this.get(i);
                this.buffer.writeInfoString(sec.name());
                sec.writeInfo();
            }
        }
        finally {
            this.buffer.endWrite();
        }
        this.buffer.endWrite();
        if (!this.buffer.writeing()) {
            this.buffer.writeEntry();
        }
    }
    
    public BufferIni(final IniBuffer aBuffer) {
        this.list = new ArrayList();
        this.buffer = aBuffer;
    }
    
    public Section get(final int index) {
        return this.list.get(index);
    }
    
    public int count() {
        return this.list.size();
    }
    
    public void clear() {
        this.list.clear();
    }
    
    public void update() {
        this.writeInfo();
    }
    
    public void eraseSection(final String section) {
        final int i = this.find(section);
        if (this.founded) {
            this.list.remove(i);
        }
    }
    
    public void eraseSection(final int index) {
        this.list.remove(index);
    }
    
    public void deleteKey(final String section, final String ident) {
        final int i = this.find(section);
        if (this.founded) {
            this.get(i).deleteKey(ident);
        }
    }
    
    public void writeNull(final String section, final String ident) {
        this.deleteKey(section, ident);
    }
    
    public void writeBoolean(final String section, final String ident, final boolean value) {
        final Section sec = this.findCreateSection(section);
        sec.writeBoolean(ident, value);
    }
    
    public void writeDouble(final String section, final String ident, final double value) {
        final Section sec = this.findCreateSection(section);
        sec.writeDouble(ident, value);
    }
    
    public void writeDateTime(final String section, final String ident, final long value) {
        final Section sec = this.findCreateSection(section);
        sec.writeDateTime(ident, value);
    }
    
    public void writeInteger(final String section, final String ident, final int value) {
        final Section sec = this.findCreateSection(section);
        sec.writeInteger(ident, value);
    }
    
    public void writeLong(final String section, final String ident, final long value) {
        final Section sec = this.findCreateSection(section);
        sec.writeLong(ident, value);
    }
    
    public void writeString(final String section, final String ident, final String value) {
        final Section sec = this.findCreateSection(section);
        sec.writeString(ident, value);
    }
    
    public void writeStream(final String section, final String ident, final Stream stream) {
        final Section sec = this.findCreateSection(section);
        sec.writeStream(ident, stream);
    }
    
    public BufferIni writeIni(final String section, final String ident) {
        final Section sec = this.findCreateSection(section);
        return sec.writeIni(ident);
    }
    
    public boolean sectionExists(final String section) {
        this.find(section);
        return this.founded;
    }
    
    public boolean valueExists(final String section, final String ident) {
        return this.findNode(section, ident) != null;
    }
    
    public boolean isNull(final String section, final String ident) {
        return !this.valueExists(section, ident);
    }
    
    public int find(final String section) {
        this.founded = false;
        int l = 0;
        int h = this.list.size() - 1;
        while (l <= h) {
            final int i = (l + h) / 2;
            final int c = this.get(i).name().compareTo(section);
            if (c < 0) {
                l = i + 1;
            }
            else {
                h = i - 1;
                if (c != 0) {
                    continue;
                }
                this.founded = true;
                l = i;
            }
        }
        return l;
    }
    
    public Section findSection(final String section) {
        final int i = this.find(section);
        if (this.founded) {
            return this.get(i);
        }
        return null;
    }
    
    public Section findCreateSection(final String section) {
        final int i = this.find(section);
        if (this.founded) {
            return this.get(i);
        }
        final Section sec = new Section(section, this.buffer);
        this.list.add(i, sec);
        return sec;
    }
    
    public Ident findNode(final String section, final String ident) {
        Ident ret = null;
        final Section sec = this.findSection(section);
        if (sec != null) {
            ret = sec.findNode(ident);
        }
        return ret;
    }
    
    public Ident findCreateNode(final String section, final String ident) {
        final Section sec = this.findCreateSection(section);
        return sec.findNodeCreate(ident);
    }
    
    public BufferIni readIni(final String section, final String ident) {
        final Section sec = this.findSection(section);
        if (sec == null) {
            return null;
        }
        return sec.readIni(ident);
    }
    
    public boolean readBoolean(final String section, final String ident, final boolean defValue) {
        final Section sec = this.findSection(section);
        boolean result;
        if (sec == null) {
            result = defValue;
        }
        else {
            result = sec.readBoolean(ident, defValue);
        }
        return result;
    }
    
    public double readDouble(final String section, final String ident, final double defValue) {
        final Section sec = this.findSection(section);
        if (sec == null) {
            return defValue;
        }
        return sec.readDouble(ident, defValue);
    }
    
    public long readDateTime(final String section, final String ident, final long defValue) {
        final Section sec = this.findSection(section);
        if (sec == null) {
            return defValue;
        }
        return sec.readDateTime(ident, defValue);
    }
    
    public int readInteger(final String section, final String ident, final int defValue) {
        final Section sec = this.findSection(section);
        if (sec == null) {
            return defValue;
        }
        return sec.readInteger(ident, defValue);
    }
    
    public long readLong(final String section, final String ident, final long defValue) {
        final Section sec = this.findSection(section);
        if (sec == null) {
            return defValue;
        }
        return sec.readLong(ident, defValue);
    }
    
    public String readString(final String section, final String ident, final String defValue) {
        final Section sec = this.findSection(section);
        if (sec == null) {
            return defValue;
        }
        return sec.readString(ident, defValue);
    }
    
    public void readStream(final String section, final String ident, final Stream stream) {
        final Section sec = this.findSection(section);
        if (sec != null) {
            sec.readStream(ident, stream);
        }
    }
    
    public void loadFromStream(final Stream aStream) {
        this.buffer.loadFromStream(aStream);
        this.readInfo();
    }
    
    public void loadFromStream(final InputStream inStream) {
        try {
            this.buffer.loadFromStream(inStream);
            this.readInfo();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void replaceStream(final Stream aStream) {
        this.buffer.replaceStream(aStream);
        this.readInfo();
    }
    
    public void saveToStream(final Stream aStream) {
        this.writeInfo();
        this.buffer.saveToStream(aStream);
    }
    
    public void saveToStream(final OutputStream outStream) {
        try {
            this.writeInfo();
            this.buffer.saveToStream(outStream);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void loadFromFile(final String fileName) {
        try {
            this.buffer.loadFromFile(fileName);
            this.readInfo();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void saveToFile(final String fileName) {
        try {
            this.update();
            this.buffer.saveToFile(fileName);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
}
